"""
Build a two object three place cogmap.
Bind two pairs, then bind the 0 object to the second place.
This is ok here, but will cause problems with retrieve on the 0 object.
However, you can notice the 1 place comes on durinig binding of the second.
"""

import pyNN.nest as sim

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from cogmapClass import cogmapBaseClass

def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)


def makeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        inputSpikeTimes3 = inputTimes[2]
        inputSpikeTimes4 = inputTimes[3]
        inputSpikeTimes5 = inputTimes[4]
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        spikeArray3 = {'spike_times': [inputSpikeTimes3]}
        spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')
        spikeArray4 = {'spike_times': [inputSpikeTimes4]}
        spikeGen4=sim.Population(1,sim.SpikeSourceArray,spikeArray4,
                                   label='inputSpikes_4')
        spikeArray5 = {'spike_times': [inputSpikeTimes5]}
        spikeGen5=sim.Population(1,sim.SpikeSourceArray,spikeArray5,
                                   label='inputSpikes_5')
        return [spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5]


#------------Main Body---------------
simName = "nest"
spinnVersion = -1

neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)

init()

spikeSources = makeSpikeSources([10.0,50.0,550.0,1050,700.0])

cogmap = cogmapBaseClass(simName,sim,neal,spinnVersion,fsa)

cogmap.createObjects(2)
cogmap.connectObjects()
cogmap.createPlaces(3)
cogmap.connectPlaces()
cogmap.setupCogMapRecording()
cogmap.makeLearningSynapses()
cogmap.connectAutomaton()

cogmap.sourceStartsAutomaton(spikeSources[0])
#Turn on one binding.  You can change to get error cases.
cogmap.sourceTurnsOnBind(spikeSources[1])
cogmap.sourceTurnsOnPlaceOn(0,spikeSources[1])
cogmap.sourceTurnsOnObjectOn(1,spikeSources[1])

cogmap.sourceTurnsOnBind(spikeSources[2])
cogmap.sourceTurnsOnPlaceOn(1,spikeSources[2])
cogmap.sourceTurnsOnObjectOn(0,spikeSources[2])

cogmap.sourceTurnsOnBind(spikeSources[3])
cogmap.sourceTurnsOnPlaceOn(2,spikeSources[3])
cogmap.sourceTurnsOnObjectOn(0,spikeSources[3])
simTime = 1500

neal.nealApplyProjections()
sim.run(simTime)

cogmap.printCogMapNets()
