"""
Using a spike source, use a bind signal, a place and an object. 
Then retrieve one of the bound objects by asking, and presenting its
mate. 
"""
import sys
import pyNN.nest as sim

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from cogmapClass import cogmapBaseClass

def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)


def makeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        inputSpikeTimes3 = inputTimes[2]
        inputSpikeTimes4 = inputTimes[3]
        inputSpikeTimes5 = inputTimes[4]
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        spikeArray3 = {'spike_times': [inputSpikeTimes3]}
        spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')
        spikeArray4 = {'spike_times': [inputSpikeTimes4]}
        spikeGen4=sim.Population(1,sim.SpikeSourceArray,spikeArray4,
                                   label='inputSpikes_4')
        spikeArray5 = {'spike_times': [inputSpikeTimes5]}
        spikeGen5=sim.Population(1,sim.SpikeSourceArray,spikeArray5,
                                   label='inputSpikes_5')
        return [spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5]


#------------Main Body---------------
simName = "nest"
spinnVersion = -1

args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 2):
    testType = int(args[1])
else:
    testType = 1
if (testType == 1):
    bindOptions = 0
    retrieveOptions = 0
elif (testType == 2):
    bindOptions = 0
    retrieveOptions = 1


neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)

init()

spikeSources = makeSpikeSources([10.0,50.0,500.0,600,700.0])

cogmap = cogmapBaseClass(simName,sim,neal,spinnVersion,fsa)

cogmap.createObjects(2)
cogmap.connectObjects()
cogmap.createPlaces(3)
cogmap.connectPlaces()
cogmap.setupCogMapRecording()
cogmap.makeLearningSynapses()
cogmap.connectAutomaton()

cogmap.sourceStartsAutomaton(spikeSources[0])
#Turn on one binding.  You can change to get error cases.
cogmap.sourceTurnsOnBind(spikeSources[1])
if (bindOptions == 0): #works
    cogmap.sourceTurnsOnPlaceOn(0,spikeSources[1])
    cogmap.sourceTurnsOnObjectOn(1,spikeSources[1])

if (retrieveOptions == 0):
    cogmap.sourceTurnOnRetrieveObjectFromPlace(spikeSources[2])
    cogmap.sourceTurnsOnPlaceQuery(spikeSources[2],0)
elif (retrieveOptions == 1):
    cogmap.sourceTurnOnRetrievePlaceFromObject(spikeSources[2])
    cogmap.sourceTurnsOnObjectQuery(spikeSources[2],1)

neal.nealApplyProjections()
sim.run(800)

cogmap.printCogMapNets()
