"""

Build a five objects five place cogmap.
It tests where the objects are and what objects have places

Criteria:
object 0 is in place 1
object 1 is in place 0 and 2
object 2 is in place 1
object 3 has no place
object 4 is in place 4
---> so, place 1 has both objects 0 and 2 
---> so, object 1 has both places 0 and 2
---> so, object 3 has no place as place 3 has no object

"""

import pyNN.nest as sim
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from cogmapClass import cogmapBaseClass

def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)


def makeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        inputSpikeTimes3 = inputTimes[2]
        inputSpikeTimes4 = inputTimes[3]
        inputSpikeTimes5 = inputTimes[4]
        inputSpikeTimes6 = inputTimes[5]
        inputSpikeTimes7 = inputTimes[6]
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        spikeArray3 = {'spike_times': [inputSpikeTimes3]}
        spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')
        spikeArray4 = {'spike_times': [inputSpikeTimes4]}
        spikeGen4=sim.Population(1,sim.SpikeSourceArray,spikeArray4,
                                   label='inputSpikes_4')
        spikeArray5 = {'spike_times': [inputSpikeTimes5]}
        spikeGen5=sim.Population(1,sim.SpikeSourceArray,spikeArray5,
                                   label='inputSpikes_5')
        spikeArray6 = {'spike_times': [inputSpikeTimes6]}
        spikeGen6=sim.Population(1,sim.SpikeSourceArray,spikeArray6,
                                   label='inputSpikes_6')
        spikeArray7 = {'spike_times': [inputSpikeTimes7]}        
        spikeGen7=sim.Population(1,sim.SpikeSourceArray,spikeArray7,
                                   label='inputSpikes_7')

        return [spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,spikeGen6,spikeGen7]


#---Main Body---
simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)

#---
init()
spikeSources = makeSpikeSources([10.0,50.0,500.0,1000.0,1500.0,2000.0,2500])

#---
cogmap = cogmapBaseClass(simName,sim,neal,spinnVersion,fsa)
cogmap.createObjects(5)
cogmap.connectObjects()
cogmap.createPlaces(5)
cogmap.connectPlaces()
cogmap.setupCogMapRecording()
cogmap.makeLearningSynapses()
cogmap.connectAutomaton()
cogmap.sourceStartsAutomaton(spikeSources[0])

# object 0 in place 1
cogmap.sourceTurnsOnBind(spikeSources[1])
cogmap.sourceTurnsOnPlaceOn(1,spikeSources[1])
cogmap.sourceTurnsOnObjectOn(0,spikeSources[1])

# object 1 in place 0
cogmap.sourceTurnsOnBind(spikeSources[2])
cogmap.sourceTurnsOnPlaceOn(0,spikeSources[2])
cogmap.sourceTurnsOnObjectOn(1,spikeSources[2])

# object 1 in place 2
cogmap.sourceTurnsOnBind(spikeSources[3])
cogmap.sourceTurnsOnPlaceOn(2,spikeSources[3])
cogmap.sourceTurnsOnObjectOn(1,spikeSources[3])

# object 2 in place 1
cogmap.sourceTurnsOnBind(spikeSources[4])
cogmap.sourceTurnsOnPlaceOn(1,spikeSources[4])
cogmap.sourceTurnsOnObjectOn(2,spikeSources[4])

# object 3 has no place as place 3 has no object

# object 4 in place 4
cogmap.sourceTurnsOnBind(spikeSources[5])
cogmap.sourceTurnsOnPlaceOn(4,spikeSources[5])
cogmap.sourceTurnsOnObjectOn(4,spikeSources[5])


###---UNCOMMENT TO TEST WHERE ARE OBJECTS OR WHAT OBJECTS HAVE PLACES

#where is the object 0? Answer: in the place 1
cogmap.sourceTurnOnRetrievePlaceFromObject(spikeSources[6])
cogmap.sourceTurnsOnObjectQuery(spikeSources[6],0)

#where is the object 1? It is assigned to both places 0 and 2 ---> the answer is only the place 2
#cogmap.sourceTurnOnRetrievePlaceFromObject(spikeSources[6])
#cogmap.sourceTurnsOnObjectQuery(spikeSources[6],1)

#where is the object 2? Answer: in the place 1
#cogmap.sourceTurnOnRetrievePlaceFromObject(spikeSources[6])
#cogmap.sourceTurnsOnObjectQuery(spikeSources[6],2)

#what objects are in place 0? Answer: only the object 1, but object 1 is also in the place 2 --> multiple answers means no answer 
#cogmap.sourceTurnOnRetrieveObjectFromPlace(spikeSources[6])
#cogmap.sourceTurnsOnPlaceQuery(spikeSources[6],0)

#what objects are in place 1? The objects 0 and 2 
#cogmap.sourceTurnOnRetrieveObjectFromPlace(spikeSources[6])
#cogmap.sourceTurnsOnPlaceQuery(spikeSources[6],1)

#what objects are in place 2? Answer: only the object 1 
#cogmap.sourceTurnOnRetrieveObjectFromPlace(spikeSources[6])
#cogmap.sourceTurnsOnPlaceQuery(spikeSources[6],2)

#what objects are in place 3? Answer: No objects are in the place 3 ---> no answer
#cogmap.sourceTurnOnRetrieveObjectFromPlace(spikeSources[6])
#cogmap.sourceTurnsOnPlaceQuery(spikeSources[6],3)

#where is the object 3? Answer: No place for the object 3 ---> no answer
#cogmap.sourceTurnOnRetrievePlaceFromObject(spikeSources[6])
#cogmap.sourceTurnsOnObjectQuery(spikeSources[6],3)

#where is the object 4? Answer: in the place 4
#cogmap.sourceTurnOnRetrievePlaceFromObject(spikeSources[6])
#cogmap.sourceTurnsOnObjectQuery(spikeSources[6],4)

#what objects are in place 4? Answer: the object 4
#cogmap.sourceTurnOnRetrieveObjectFromPlace(spikeSources[6])
#cogmap.sourceTurnsOnPlaceQuery(spikeSources[6],4)



#---
neal.nealApplyProjections()
simTime = 3000
sim.run(simTime)
cogmap.printCogMapNets()
