"""
Test the binding of one object place pair.   
Check by turning on a place with the object coming on because their bound.
"""

import pyNN.spiNNaker as sim

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from cogmapClass import cogmapBaseClass

def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)

def makeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        inputSpikeTimes3 = inputTimes[2]
        inputSpikeTimes4 = inputTimes[3]

        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        spikeArray3 = {'spike_times': [inputSpikeTimes3]}
        spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')
        spikeArray4 = {'spike_times': [inputSpikeTimes4]}
        spikeGen4=sim.Population(1,sim.SpikeSourceArray,spikeArray4,
                                   label='inputSpikes_4')
        return [spikeGen1,spikeGen2,spikeGen3,spikeGen4]


#------------Main Body---------------
simName = "spinnaker"
spinnVersion = 8

neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)

init()
sim.set_number_of_neurons_per_core(sim.IF_cond_exp,10)

spikeSources = makeSpikeSources([10.0,50.0,150.0,200])

cogmap = cogmapBaseClass(simName,sim,neal,spinnVersion,fsa)

cogmap.createObjects(2)
cogmap.createPlaces(3)
cogmap.setupCogMapRecording()
cogmap.makeLearningSynapses()

cogmap.sourceTurnsOnPlace(0,spikeSources[0])
cogmap.sourceTurnsOnObject(1,spikeSources[1])
cogmap.sourceTurnsOffPlace(0,spikeSources[2])
cogmap.sourceTurnsOffObject(1,spikeSources[2])

#choose one of these four for the test
cogmap.sourceTurnsOnPlace(0,spikeSources[3])  #place then object after 200
#cogmap.sourceTurnsOnObject(1,spikeSources[3]) #object then place after 200
#cogmap.sourceTurnsOnPlace(2,spikeSources[3])  #just place 2 after 200
#cogmap.sourceTurnsOnObject(0,spikeSources[3])  #just object 0 after 200


neal.nealApplyProjections()
sim.run(400)

cogmap.printCogMapNets()
sim.end()
