"""
Build a two object three place cogmap.
There are three ways to fail.
Call this with the way you'd like.
The first is to have two places. 
The second is to have two objects.
The third is to have one object and no places (or vice versa).
"""
import sys

import pyNN.spiNNaker as sim

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from cogmapClass import cogmapBaseClass

def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)


def makeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        return [spikeGen1,spikeGen2]


#------------Main Body---------------
simName = "spinnaker"
spinnVersion = 8

args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 2):
    testType = int(args[1])
else:
    testType = 3


neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)

init()
sim.set_number_of_neurons_per_core(sim.IF_cond_exp,10)

spikeSources = makeSpikeSources([10.0,50.0])

cogmap = cogmapBaseClass(simName,sim,neal,spinnVersion,fsa)

cogmap.createObjects(2)
cogmap.connectObjects()
cogmap.createPlaces(3)
cogmap.connectPlaces()
cogmap.setupCogMapRecording()
cogmap.makeLearningSynapses()
cogmap.connectAutomaton()

cogmap.sourceStartsAutomaton(spikeSources[0])
#Turn on one binding.  You can change to get error cases.
cogmap.sourceTurnsOnBind(spikeSources[1])
if (testType == 1):
    cogmap.sourceTurnsOnPlaceOn(0,spikeSources[1])
    cogmap.sourceTurnsOnPlaceOn(1,spikeSources[1])
    simTime = 600
elif (testType == 2):
    cogmap.sourceTurnsOnObjectOn(0,spikeSources[1])
    cogmap.sourceTurnsOnObjectOn(1,spikeSources[1])
    simTime = 600
elif (testType == 3):
    cogmap.sourceTurnsOnObjectOn(0,spikeSources[1])
    simTime = 600

neal.nealApplyProjections()
sim.run(simTime)

cogmap.printCogMapNets()
sim.end()
